/* $Id: iterator.h,v 1.9 1999/01/20 07:57:00 marek Exp $ */
/*
** Copyright (C) 1999 Marek Rychlik <rychlik@u.arizona.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#ifndef _EXAMPLE_H_
#define _EXAMPLE_H_

#include <math.h>
#include <stdarg.h>
/*
 * This file is intended to be included by any
 * file that defines a new Iterator example
 */

/*
 * structure defining a point in user coordinates
 */
typedef struct point {double x,y;} Point;

/*
 * The type of a map (dynamical system)
 * The first argument is the argument point
 * The second argument is the image 
 * The third argument is the pointer to the parameters
 */
typedef int (*Map)(Point *, Point *, double *);

/* 
 * Various useful definitions
 */
#define PI		M_PI	/* pi */
#define TWO_PI		(2*M_PI) /* 2 pi */
#define GOLDEN_RATIO	(.61803398874989484820)

/*
 * Many examples require the fractional part to be extracted
 * from a floating point number. This definition does the job
 * in a portable fashion
 */
#define FRAC(x)		((x)-floor(x))


/*
 * Parameter description
 */
typedef struct parameter_t {
  double initv;			/* initial value */
  double minv;			/* minimal value */
  double maxv;			/* maximal value */
  char *name;			/* user name */
} parameter;

typedef struct parameter_data_t {
  int pno;
  parameter *p;
} parameter_data;

/* 
 * Phase space of an example represented by square. 
 */
typedef enum Space_t {
  PLANE=0,
  CYLINDER=1,
  TORUS=2,
} Space;

/*
 * Origin and basis of the initial coordinate system
 */
typedef struct basis_data_t {
  double origin_x, origin_y, e_x, e_y, f_x, f_y;
} basis_data;

/*
 * A structure to represent an example
 */
typedef struct example_data_t {
  Space space;				 /* phase space */
  char *label;				 /* label */
  char *var_x;				 /* Variable names */
  char *var_y;
  char *fmap_x;				 /* Forward map formula */
  char *fmap_y;				 
  char *bmap_x;				 /* Inverse map formula */
  char *bmap_y;
  Map fmap;				 /* forward map */
  Map bmap;				 /* backward map */
  basis_data basis;			 /* affine basis */
  char *directory;			 /* working directory */
  char *cfilename;			 /* C source filename */
  char *sofilename;			 /* Shared object name */
  char *doc;				 /* documentation file */
  char *linker_opts;			 /* extra arguments to the linker */
  char *comp_options;			 /* extra compiler options */
  char *includes;			 /* extra include files */
  parameter_data *pdata;		 /* parameter data */
  double *par;				 /* current parameter settings */
  void *shlib;			         /* example shared library handle */
  int id;				 /* index into private table */
  long client_data;			 /* a user hook */
} example_data;

/*
 * The actual number of examples. 
 */
extern int example_no(void);

/*
 * Return a pointer to example i
 */
extern example_data *example(int i);

/*
 * Pointer to an error function called by the example package
 */
extern void (* example_error_func)(char *fmt, ...);

/* 
 * The default error function: it has the calling sequence of
 * printf and sends its output to stderr
 */
extern void default_example_error_func(char *fmt, ...);

/*
 * A constructor of parameter lists
 */
extern parameter_data *create_pdata(int pno, ...);
extern parameter_data *resize_pdata(int pno, parameter_data *orig);

/*
 * A destructor of parameter lists
 */
void destroy_pdata(parameter_data *pdata);

/*
 * A constructor of examples; useful for statically linked
 * examples for which the formulas are not available, and
 * thus editing of formulas is not provided.
 */
extern example_data *
create_example(Space space,
	       char *label,
	       char *var_x,     char *var_y,
	       char *fmap_x,    char *fmap_y,
	       char *bmap_x,    char *bmap_y,
	       Map fmap,        Map bmap,
	       double origin_x, double origin_y,
	       double e_x, double e_y,
	       double f_x, double f_y,
	       char *directory,
	       char *cfilename,
	       char *sofilename,
	       char *doc,
	       int pno,
	       ...);

/*
 * Make a copy of an example
 * However, if the original example uses a shared object,
 * forget the references to it.
 */
extern example_data *copy_example(example_data *orig);


/*
 * Add new example; fails, if memory for new table entry cannot be allocated
 * Returns example id number or -1 upon failure
 */
extern int add_example(example_data *ex);

/*
 * Replace an existing example number n with a new one
 */
extern int replace_example(example_data *ex, int n);

/*
 * Returns a pointer to private data that should
 * not be edited
 */
extern char *example_label(int n);

/*
 * Example destructor
 */
extern void destroy_example(example_data *ep);

/*
 * Load example from shared object file
 */
extern example_data *load_example(char *shlibname);

/*
 * Install a built-in example
 */
example_data * 
new_example(Space space,
	    char *label,
	    char *var_x,     char *var_y,
	    char *fmap_x,    char *fmap_y,
	    char *bmap_x,    char *bmap_y,
	    Map fmap,        Map bmap,
	    double origin_x, double origin_y,
	    double e_x, double e_y,
	    double f_x, double f_y,
	    char *doc,
	    int pno,
	    ...);

/*
 * Utility: Make a copy of a string
 */
extern char *copy_string(char *s);

/*
 * Copy string s to the documentation slot of example
 */
extern int set_documentation(example_data *ep, char *s);

/*
 * Set a  comma-separated list of options to the linker
 * and compiler to be used with a newly created example
 */
extern void set_default_linker_options(char *fmt, ...);
extern void set_default_compiler_options(char *fmt, ...);


#endif
